//DJS 5_17_16

`timescale 1ps/1ps
module pulse_generator
(
	clk,
	sync,
	csb,
	cnvclk,
	sck_gate,
	rx_start,
	num_bits
);

input clk,sync,num_bits;
output csb,cnvclk,sck_gate,rx_start;

reg csb,cnvclk,sck_gate,rx_start;
reg [7:0] sck_length,tsckgateh;
reg [6:0] count;						// master counter
//this register resets to '0' while the 'sync' pulse from the PLL is 'high'.
//the following pulses are timed by this count: 'csb', 'sck_gate', 'rx_start', 'cnvclk_gate'

wire [5:0] num_bits;

parameter [6:0] tcsbh = 7'd0;  					// Time for 'csb' to go high; 2 clock cycles after 'sync' rising edge
parameter [6:0] csb_length = 7'd3;				// Yields 'CNV' pulsewidth of 47nsec
parameter [6:0] tcnvclkh = 7'd2;  				// Time for 'cnvclk' to go high; 4 clock cycles after 'sync' rising edge
parameter [6:0] cnvclk_length = 7'd2;			// Yields 'CNVCLK' pulsewidth of 25nsec
parameter [6:0] trx_starth = 7'd10;  			// Time for 'rx_start' to go high; 12 clock cycles after 'sync' rising edge
parameter [6:0] rx_start_length = 7'd2;	   // Yields 'rx_start' pulsewidth of 25nsec

initial
begin
  count = 0;
  tsckgateh = 7'd8;			// Time for 'sck_gate' to go high; 10 clock cycles after 'sync' rising edge
									// Yields first 'SCK' edge 68nsec after 'CNV' falls
end

always @ (clk)  //decode the number of bits
begin
case(num_bits)
	5'd16:
		sck_length = 7'd32;
	5'd14:
		sck_length = 7'd32;
	5'd12:
		sck_length = 7'd28;
	default:
		sck_length = 7'd32;
endcase
end

always @ (posedge clk)
    if (sync)
		  count <= 7'd0;
	 else 
		  count <= count + 7'd1;
		  	
//generate 'csb' pulse
always @ (posedge clk)
	if (count == tcsbh)
		csb <= 1'b1;
	else if (count == tcsbh + csb_length)
		csb <= 1'b0;
	else
		csb <= csb;	

//generate 'sck_gate' pulse		
always @ (posedge clk)
	if (count == tsckgateh)
		sck_gate <= 1'b1;
	else if (count == tsckgateh + sck_length)
		sck_gate <= 1'b0;
	else
		sck_gate <= sck_gate;
			
//generate 'rx_start' pulse	
always @ (posedge clk)
	if (count == trx_starth)
		rx_start <= 1'b1;
	else if (count == trx_starth + rx_start_length)
		rx_start <= 1'b0;
	else
		rx_start <= rx_start;

//generate 'cnvclk' pulse		
always @ (posedge clk)
	if (count == tcnvclkh)
		cnvclk <= 1'b1;
	else if (count == tcnvclkh + cnvclk_length)
		cnvclk <= 1'b0;
	else
		cnvclk <= cnvclk;	

endmodule